/*
 * merge.cu
 * 
 * Tests  Merge Sort Implementation run in parallel threads
 *   on the GPU through the nVidia CUDA Framework
 * 
 * Jim Kukunas and James Devine
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <cutil_inline.h>
#include "merge_kernel.cu"

int main(int argc, char** argv)
{
	if( cutCheckCmdLineFlag(argc, (const char**)argv, "device") )
		cutilDeviceInit(argc, argv);
	else
		cudaSetDevice( cutGetMaxGflopsDeviceId() );

    int values[NUM];

    /* initialize a random dataset */
    for(int i = 0; i < NUM; i++)
    {
        values[i] = rand();
    }

    int* dvalues,
       * results;
    cutilSafeCall(cudaMalloc((void**)&dvalues, sizeof(int) * NUM));
    cutilSafeCall(cudaMemcpy(dvalues, values, sizeof(int) * NUM, cudaMemcpyHostToDevice));
    cutilSafeCall(cudaMalloc((void**)&results, sizeof(int) * NUM));
    cutilSafeCall(cudaMemcpy(results,values , sizeof(int)* NUM, cudaMemcpyHostToDevice));

    MergeSort<<<1, NUM, sizeof(int) * NUM*2>>>(dvalues, results);

    // check for any errors
    cutilCheckMsg("Kernel execution failed");

    cutilSafeCall(cudaFree(dvalues));
    cutilSafeCall(cudaMemcpy(values, results, sizeof(int)*NUM, cudaMemcpyDeviceToHost));
    cutilSafeCall(cudaFree(results));

    bool passed = true;
    for(int i = 1; i < NUM; i++)
    {
        if (values[i-1] > values[i])
        {
            passed = false;
        }
    }
    printf( "Test %s\n", passed ? "PASSED" : "FAILED");
    cudaThreadExit();
    cutilExit(argc, argv);  
}
